// ### HELPER FUNCTIONS

// merge attributes from arr into base
var _merge = function (base, arr) {
    for (var k in base) {
        if (arr[k] !== undefined) {
            base[k] = arr[k];
        }
    }
    return base;
};

// bool to str
var _bool = function(arg) {
    return arg ? 'true' : 'false';
};

// join all arguments
var _ret = function() {
    return [].join.call(arguments, '');
};

// output str if o is null or false
var _n = function(o, str) {
    return (o === null || o === false) ? str : '';
};

// output str if o is not null or false
var _nn = function(o, str) {
    return (o !== null && o !== false) ? str : '';
};

// is arg a string?
var _isStr = function(arg) {
    return typeof arg == 'string';
}


// ### BOOTSTRAP COMPONENT GENERATORS

// Glpyhicon
exports.Glyphicon = function(o) {
    if (_isStr(o)) {
        o = { icon: o };
    }
    o = _merge({
        icon: null,
        ariaHidden: true,
        ariaLabel: null,
    }, o);

    return _ret(
        '<span class="glyphicon glyphicon-' + o.icon + '" aria-hidden="' + _bool(o.ariaHidden) + '"',
        _nn(o.ariaLabel, ' aria-label="' + o.ariaLabel + '"'),
        '></span>'
    );
};

// Dropdown
exports.Dropdown = function(o) {
    o = _merge({
        dropdownWrap: 'div',
        alignRight: false,
        id: null,
        label: null,
        items: [],
    }, o);

    var items = '';
    if (o.items) {
        for (var i in o.items) {
            items += exports.DropdownItem(o.items[i]);
        }
    }

    return _ret(
        '<' + o.dropdownWrap + ' class="dropdown">',
        '<button class="btn btn-default dropdown-toggle" type="button" id="' + o.id + '" data-toggle="dropdown" aria-expanded="true">',
        o.label + '<span class="caret"></span></button><ul class="dropdown-menu' + _nn(o.alignRight, ' dropdown-menu-right'),
        '" role="menu" aria-labelledby="' + o.id + '">' + items + '</ul></' + o.dropdownWrap + '>'
    );
};

// Dropdown Item
exports.DropdownItem = function(o) {
    if (o == '-') {
        return '<li role="presentation" class="divider"></li>';
    } else if (_isStr(o)) {
        return o;
    }

    o = _merge({
        items: null,
        id: null,
        href: null,
        label: null,
        disabled: false,
    }, o);

    if (o.items) {
        var items = '';
        for (var i in o.items) {
            items += exports.DropdownItem(o.items[i]);
        }
        return '<li role="presentation" class="dropdown-header">' + o.label + '</li>' + items;
    }

    return _ret(
        '<li role="presentation"' + _nn(o.disabled, ' class="disabled"') + '><a role="menuitem" tabindex="-1"',
        _nn(o.id, ' id="' + o.id + '"'),
        _nn(o.href, ' href="' + o.href + '"'),
        '>' + o.label + '</a></li>'
    );
};

// Embed (responsive)
exports.beginEmbed = function(o) {
    switch (o) {
        case '16:9':
        case '16/9':
        case '16by9': o = 'embed-responsive-16by9'; break;
        case '4:3':
        case '4/3':
        case '4by3': o = 'embed-responsive-4by3'; break;
        default: o = null;
    }

    return '<div class="embed-responsive' + _nn(o, ' ' + o) + '">';
};
exports.endEmbed = function() {
    return '</div>';
};

// Well
exports.beginWell = function(o) {
    switch (o) {
        case 'large':
        case 'lg': o = 'well-lg'; break;
        case 'small':
        case 'sm': o = 'well-sm'; break;
        default: o = null;
    }

    return '<div class="well' + _nn(o, ' ' + o) + '">';
};
exports.endWell = function() {
    return '</div>';
};
